@@ -25,7 +25,7 @@
  *  as some of the InstallWatch code was used.
  *
  *
- *  $Header: /var/cvsroot/gentoo-src/sandbox/libsandbox.c,v 1.24 2005/05/13 15:23:28 azarah Exp $
+ *  $Header$
  *
  */
 
@@ -86,36 +86,41 @@
 #define FUNCTION_SANDBOX_SAFE(func, path) \
 	((0 == is_sandbox_on()) || (1 == before_syscall(func, path)))
 
-#define FUNCTION_SANDBOX_SAFE_INT(func, path, flags) \
+#define FUNCTION_SANDBOX_SAFE_ACCESS(func, path, flags) \
+	((0 == is_sandbox_on()) || (1 == before_syscall_access(func, path, flags)))
+
+#define FUNCTION_SANDBOX_SAFE_OPEN_INT(func, path, flags) \
 	((0 == is_sandbox_on()) || (1 == before_syscall_open_int(func, path, flags)))
 
-#define FUNCTION_SANDBOX_SAFE_CHAR(func, path, mode) \
+#define FUNCTION_SANDBOX_SAFE_OPEN_CHAR(func, path, mode) \
 	((0 == is_sandbox_on()) || (1 == before_syscall_open_char(func, path, mode)))
 
 /* Macro to check if a wrapper is defined, if not
  * then try to resolve it again. */
 #define check_dlsym(name) \
 { \
-  int old_errno=errno; \
-  if (!true_ ## name) \
-	true_ ## name = get_dlsym(symname_ ## name, symver_ ## name); \
-  errno=old_errno; \
+	int old_errno=errno; \
+	if (!true_ ## name) \
+		true_ ## name = get_dlsym(symname_ ## name, symver_ ## name); \
+	errno=old_errno; \
 }
 
 /* Macro to check if we could canonicalize a path.  It returns an integer on
  * failure. */
 #define canonicalize_int(path, resolved_path) \
 { \
-  if (0 != canonicalize(path, resolved_path)) \
-    return -1; \
+	/* Do not fail if the canonicalized path is too long, bug #94630 */ \
+	if (0 != canonicalize(path, resolved_path, 0)) \
+		return -1; \
 }
 
 /* Macro to check if we could canonicalize a path.  It returns a NULL pointer on
  * failure. */
 #define canonicalize_ptr(path, resolved_path) \
 { \
-  if (0 != canonicalize(path, resolved_path)) \
-    return NULL; \
+	/* Do not fail if the canonicalized path is too long, bug #94630 */ \
+	if (0 != canonicalize(path, resolved_path, 0)) \
+		return NULL; \
 }
 
 static char sandbox_lib[SB_PATH_MAX];
@@ -150,11 +155,12 @@ static char *egetcwd(char *, size_t);
 
 static void init_wrappers(void);
 static void *get_dlsym(const char *, const char *);
-static int canonicalize(const char *, char *);
+static int canonicalize(const char *, char *, int);
 static char *filter_path(const char *, int);
 static int check_access(sbcontext_t *, const char *, const char *, const char *);
 static int check_syscall(sbcontext_t *, const char *, const char *);
 static int before_syscall(const char *, const char *);
+static int before_syscall_access(const char *, const char *, int);
 static int before_syscall_open_int(const char *, const char *, int);
 static int before_syscall_open_char(const char *, const char *, const char *);
 static void clean_env_entries(char ***, int *);
@@ -166,48 +172,50 @@ static int is_sandbox_pid();
 /* Wrapped functions */
 
 extern int chmod(const char *, mode_t);
-static int (*true_chmod) (const char *, mode_t);
+static int (*true_chmod) (const char *, mode_t) = NULL;
 extern int chown(const char *, uid_t, gid_t);
-static int (*true_chown) (const char *, uid_t, gid_t);
+static int (*true_chown) (const char *, uid_t, gid_t) = NULL;
 extern int creat(const char *, mode_t);
-static int (*true_creat) (const char *, mode_t);
+static int (*true_creat) (const char *, mode_t) = NULL;
 extern FILE *fopen(const char *, const char *);
-static FILE *(*true_fopen) (const char *, const char *);
+static FILE *(*true_fopen) (const char *, const char *) = NULL;
 extern int lchown(const char *, uid_t, gid_t);
-static int (*true_lchown) (const char *, uid_t, gid_t);
+static int (*true_lchown) (const char *, uid_t, gid_t) = NULL;
 extern int link(const char *, const char *);
-static int (*true_link) (const char *, const char *);
+static int (*true_link) (const char *, const char *) = NULL;
 extern int mkdir(const char *, mode_t);
-static int (*true_mkdir) (const char *, mode_t);
+static int (*true_mkdir) (const char *, mode_t) = NULL;
 extern DIR *opendir(const char *);
-static DIR *(*true_opendir) (const char *);
+static DIR *(*true_opendir) (const char *) = NULL;
 #ifdef WRAP_MKNOD
 extern int __xmknod(const char *, mode_t, dev_t);
-static int (*true___xmknod) (const char *, mode_t, dev_t);
+static int (*true___xmknod) (const char *, mode_t, dev_t) = NULL;
 #endif
+extern int access(const char *, int);
+static int (*true_access) (const char *, int) = NULL;
 extern int open(const char *, int, ...);
-static int (*true_open) (const char *, int, ...);
+static int (*true_open) (const char *, int, ...) = NULL;
 extern int rename(const char *, const char *);
-static int (*true_rename) (const char *, const char *);
+static int (*true_rename) (const char *, const char *) = NULL;
 extern int rmdir(const char *);
-static int (*true_rmdir) (const char *);
+static int (*true_rmdir) (const char *) = NULL;
 extern int symlink(const char *, const char *);
-static int (*true_symlink) (const char *, const char *);
+static int (*true_symlink) (const char *, const char *) = NULL;
 extern int truncate(const char *, TRUNCATE_T);
-static int (*true_truncate) (const char *, TRUNCATE_T);
+static int (*true_truncate) (const char *, TRUNCATE_T) = NULL;
 extern int unlink(const char *);
-static int (*true_unlink) (const char *);
+static int (*true_unlink) (const char *) = NULL;
 
 #if (GLIBC_MINOR >= 1)
 
 extern int creat64(const char *, __mode_t);
-static int (*true_creat64) (const char *, __mode_t);
+static int (*true_creat64) (const char *, __mode_t) = NULL;
 extern FILE *fopen64(const char *, const char *);
-static FILE *(*true_fopen64) (const char *, const char *);
+static FILE *(*true_fopen64) (const char *, const char *) = NULL;
 extern int open64(const char *, int, ...);
-static int (*true_open64) (const char *, int, ...);
+static int (*true_open64) (const char *, int, ...) = NULL;
 extern int truncate64(const char *, __off64_t);
-static int (*true_truncate64) (const char *, __off64_t);
+static int (*true_truncate64) (const char *, __off64_t) = NULL;
 
 #endif
 
@@ -231,6 +239,7 @@ static void init_wrappers(void)
 #ifdef WRAP_MKNOD
 	check_dlsym(__xmknod);
 #endif
+	check_dlsym(access);
 	check_dlsym(open);
 	check_dlsym(rename);
 	check_dlsym(rmdir);
@@ -326,7 +335,13 @@ void __attribute__ ((constructor)) libsb_init(void)
 	errno = old_errno;
 }
 
-static int canonicalize(const char *path, char *resolved_path)
+/* Be default we will fail if the path name we try to canonicalize is too long.
+ * This however could cause issues with some things (bug #94630 and #21766), so
+ * if fail_nametoolong == 0, return a null length string and do not fail.
+ * FIXME:  This is really not *safe* if you belong to the sandbox should avoid
+ *         any exploits that might want to touch the fs during compile ...
+ */
+static int canonicalize(const char *path, char *resolved_path, int fail_nametoolong)
 {
 	int old_errno = errno;
 	char *retval;
@@ -352,10 +367,26 @@ static int canonicalize(const char *path, char *resolved_path)
 		 * to the current working directory if it was not
 		 * an absolute path
 		 */
-		if (errno == ENAMETOOLONG)
-			return -1;
+		
+		if (ENAMETOOLONG == errno) {
+			if (0 == fail_nametoolong) {
+				errno = 0;
+				*resolved_path = '\0';
+				return 0;
+			} else {
+				return -1;
+			}
+		}
 
-		egetcwd(resolved_path, SB_PATH_MAX - 2);
+		if (NULL == egetcwd(resolved_path, SB_PATH_MAX - 2)) {
+			if (ENAMETOOLONG == errno && 0 == fail_nametoolong) {
+				errno = 0;
+				*resolved_path = '\0';
+				return 0;
+			} else {
+				return -1;
+			}
+		}
 		strcat(resolved_path, "/");
 		strncat(resolved_path, path, SB_PATH_MAX - 1);
 
@@ -396,7 +427,7 @@ static char *filter_path(const char *path, int follow_link)
 		return NULL;
 
 	if (0 == follow_link) {
-		if (-1 == canonicalize(path, filtered_path))
+		if (-1 == canonicalize(path, filtered_path, 1))
 			goto error;
 	} else {
 		/* Basically we get the realpath which should resolve symlinks,
@@ -414,7 +445,7 @@ static char *filter_path(const char *path, int follow_link)
 			 * parent directory */
 			if (NULL == realpath(dname, filtered_path)) {
 				/* Fall back to canonicalize */
-				if (-1 == canonicalize(path, filtered_path)) {
+				if (-1 == canonicalize(path, filtered_path, 1)) {
 					free(tmp_str1);
 					goto error;
 				}
@@ -461,11 +492,10 @@ int chmod(const char *path, mode_t mode)
 
 	canonicalize_int(path, canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("chmod", canonic) {
+	if FUNCTION_SANDBOX_SAFE("chmod", canonic) {
 		check_dlsym(chmod);
 		result = true_chmod(path, mode);
-		}
+	}
 
 	return result;
 }
@@ -477,11 +507,10 @@ int chown(const char *path, uid_t owner, gid_t group)
 
 	canonicalize_int(path, canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("chown", canonic) {
+	if FUNCTION_SANDBOX_SAFE("chown", canonic) {
 		check_dlsym(chown);
 		result = true_chown(path, owner, group);
-		}
+	}
 
 	return result;
 }
@@ -494,11 +523,10 @@ int creat(const char *pathname, mode_t mode)
 
 	canonicalize_int(pathname, canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("creat", canonic) {
+	if FUNCTION_SANDBOX_SAFE("creat", canonic) {
 		check_dlsym(open);
 		result = true_open(pathname, O_CREAT | O_WRONLY | O_TRUNC, mode);
-		}
+	}
 
 	return result;
 }
@@ -510,11 +538,10 @@ FILE *fopen(const char *pathname, const char *mode)
 
 	canonicalize_ptr(pathname, canonic);
 
-	if FUNCTION_SANDBOX_SAFE_CHAR
-		("fopen", canonic, mode) {
+	if FUNCTION_SANDBOX_SAFE_OPEN_CHAR("fopen", canonic, mode) {
 		check_dlsym(fopen);
 		result = true_fopen(pathname, mode);
-		}
+	}
 
 	return result;
 }
@@ -526,11 +553,10 @@ int lchown(const char *path, uid_t owner, gid_t group)
 
 	canonicalize_int(path, canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("lchown", canonic) {
+	if FUNCTION_SANDBOX_SAFE("lchown", canonic) {
 		check_dlsym(lchown);
 		result = true_lchown(path, owner, group);
-		}
+	}
 
 	return result;
 }
@@ -543,11 +569,10 @@ int link(const char *oldpath, const char *newpath)
 	canonicalize_int(oldpath, old_canonic);
 	canonicalize_int(newpath, new_canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("link", new_canonic) {
+	if FUNCTION_SANDBOX_SAFE("link", new_canonic) {
 		check_dlsym(link);
 		result = true_link(oldpath, newpath);
-		}
+	}
 
 	return result;
 }
@@ -569,11 +594,10 @@ int mkdir(const char *pathname, mode_t mode)
 	}
 	errno = my_errno;
 
-	if FUNCTION_SANDBOX_SAFE
-		("mkdir", canonic) {
+	if FUNCTION_SANDBOX_SAFE("mkdir", canonic) {
 		check_dlsym(mkdir);
 		result = true_mkdir(pathname, mode);
-		}
+	}
 
 	return result;
 }
@@ -585,11 +609,10 @@ DIR *opendir(const char *name)
 
 	canonicalize_ptr(name, canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("opendir", canonic) {
+	if FUNCTION_SANDBOX_SAFE("opendir", canonic) {
 		check_dlsym(opendir);
 		result = true_opendir(name);
-		}
+	}
 
 	return result;
 }
@@ -603,17 +626,31 @@ int __xmknod(const char *pathname, mode_t mode, dev_t dev)
 
 	canonicalize_int(pathname, canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("__xmknod", canonic) {
+	if FUNCTION_SANDBOX_SAFE("__xmknod", canonic) {
 		check_dlsym(__xmknod);
 		result = true___xmknod(pathname, mode, dev);
-		}
+	}
 
 	return result;
 }
 
 #endif
 
+int access(const char *pathname, int mode)
+{
+	int result = -1;
+	char canonic[SB_PATH_MAX];
+
+	canonicalize_int(pathname, canonic);
+
+	if FUNCTION_SANDBOX_SAFE_ACCESS("access", canonic, mode) {
+		check_dlsym(access);
+		result = true_access(pathname, mode);
+	}
+
+	return result;
+}
+
 /* Eventually, there is a third parameter: it's mode_t mode */
 int open(const char *pathname, int flags, ...)
 {
@@ -630,11 +667,10 @@ int open(const char *pathname, int flags, ...)
 
 	canonicalize_int(pathname, canonic);
 
-	if FUNCTION_SANDBOX_SAFE_INT
-		("open", canonic, flags) {
+	if FUNCTION_SANDBOX_SAFE_OPEN_INT("open", canonic, flags) {
 		check_dlsym(open);
 		result = true_open(pathname, flags, mode);
-		}
+	}
 
 	return result;
 }
@@ -663,11 +699,10 @@ int rmdir(const char *pathname)
 
 	canonicalize_int(pathname, canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("rmdir", canonic) {
+	if FUNCTION_SANDBOX_SAFE("rmdir", canonic) {
 		check_dlsym(rmdir);
 		result = true_rmdir(pathname);
-		}
+	}
 
 	return result;
 }
@@ -680,11 +715,10 @@ int symlink(const char *oldpath, const char *newpath)
 	canonicalize_int(oldpath, old_canonic);
 	canonicalize_int(newpath, new_canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("symlink", new_canonic) {
+	if FUNCTION_SANDBOX_SAFE("symlink", new_canonic) {
 		check_dlsym(symlink);
 		result = true_symlink(oldpath, newpath);
-		}
+	}
 
 	return result;
 }
@@ -696,11 +730,10 @@ int truncate(const char *path, TRUNCATE_T length)
 
 	canonicalize_int(path, canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("truncate", canonic) {
+	if FUNCTION_SANDBOX_SAFE("truncate", canonic) {
 		check_dlsym(truncate);
 		result = true_truncate(path, length);
-		}
+	}
 
 	return result;
 }
@@ -720,11 +753,10 @@ int unlink(const char *pathname)
 		return result;
 	}
 
-	if FUNCTION_SANDBOX_SAFE
-		("unlink", canonic) {
+	if FUNCTION_SANDBOX_SAFE("unlink", canonic) {
 		check_dlsym(unlink);
 		result = true_unlink(pathname);
-		}
+	}
 
 	return result;
 }
@@ -739,11 +771,10 @@ int creat64(const char *pathname, __mode_t mode)
 
 	canonicalize_int(pathname, canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("creat64", canonic) {
+	if FUNCTION_SANDBOX_SAFE("creat64", canonic) {
 		check_dlsym(open64);
 		result = true_open64(pathname, O_CREAT | O_WRONLY | O_TRUNC, mode);
-		}
+	}
 
 	return result;
 }
@@ -755,11 +786,10 @@ FILE *fopen64(const char *pathname, const char *mode)
 
 	canonicalize_ptr(pathname, canonic);
 
-	if FUNCTION_SANDBOX_SAFE_CHAR
-		("fopen64", canonic, mode) {
+	if FUNCTION_SANDBOX_SAFE_OPEN_CHAR("fopen64", canonic, mode) {
 		check_dlsym(fopen64);
 		result = true_fopen64(pathname, mode);
-		}
+	}
 
 	return result;
 }
@@ -780,11 +810,10 @@ int open64(const char *pathname, int flags, ...)
 
 	canonicalize_int(pathname, canonic);
 
-	if FUNCTION_SANDBOX_SAFE_INT
-		("open64", canonic, flags) {
+	if FUNCTION_SANDBOX_SAFE_OPEN_INT("open64", canonic, flags) {
 		check_dlsym(open64);
 		result = true_open64(pathname, flags, mode);
-		}
+	}
 
 	return result;
 }
@@ -796,11 +825,10 @@ int truncate64(const char *path, __off64_t length)
 
 	canonicalize_int(path, canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("truncate64", canonic) {
+	if FUNCTION_SANDBOX_SAFE("truncate64", canonic) {
 		check_dlsym(truncate64);
 		result = true_truncate64(path, length);
-		}
+	}
 
 	return result;
 }
@@ -821,12 +849,11 @@ int execve(const char *filename, char *const argv[], char *const envp[])
 	char **my_env = NULL;
 	int kill_env = 1;
 	/* We limit the size LD_PRELOAD can be here, but it should be enough */
-	char tmp_str[4096];
+	char tmp_str[SB_BUF_LEN];
 
 	canonicalize_int(filename, canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("execve", canonic) {
+	if FUNCTION_SANDBOX_SAFE("execve", canonic) {
 		while (envp[count] != NULL) {
 			/* Check if we do not have to do anything */
 			if (strstr(envp[count], LD_PRELOAD_EQ) == envp[count]) {
@@ -848,7 +875,7 @@ int execve(const char *filename, char *const argv[], char *const envp[])
 				const int max_envp_len = strlen(envp[count]) + strlen(sandbox_lib) + 1;
 
 				/* Fail safe ... */
-				if (max_envp_len > 4096) {
+				if (max_envp_len > SB_BUF_LEN) {
 					fprintf(stderr, "sandbox:  max_envp_len too big!\n");
 					errno = ENOMEM;
 					return result;
@@ -902,18 +929,15 @@ int execve(const char *filename, char *const argv[], char *const envp[])
 			count++;
 		}
 
-	      end_loop:
+		end_loop:
 		errno = old_errno;
 		check_dlsym(execve);
 		result = true_execve(filename, argv, my_env);
 		old_errno = errno;
 
-		if (my_env && kill_env) {
-			free(tmp_str);
+		if (my_env && kill_env)
 			free(my_env);
-			my_env = NULL;
-		}
-		}
+	}
 
 	errno = old_errno;
 
@@ -968,6 +992,7 @@ static int is_sandbox_pid()
 
 	init_wrappers();
 
+	check_dlsym(fopen);
 	pids_stream = true_fopen(sandbox_pids_file, "r");
 
 	if (NULL == pids_stream) {
@@ -1150,7 +1175,8 @@ static int check_access(sbcontext_t * sbcontext, const char *func, const char *p
 
 	if (-1 == result) {
 		if ((NULL != sbcontext->read_prefixes) &&
-		    ((0 == strncmp(func, "open_rd", 7)) ||
+		    ((0 == strncmp(func, "access_rd", 9)) ||
+		     (0 == strncmp(func, "open_rd", 7)) ||
 		     (0 == strncmp(func, "popen", 5)) ||
 		     (0 == strncmp(func, "opendir", 7)) ||
 		     (0 == strncmp(func, "system", 6)) ||
@@ -1172,7 +1198,8 @@ static int check_access(sbcontext_t * sbcontext, const char *func, const char *p
 			}
 		}
 		if ((NULL != sbcontext->write_prefixes) &&
-		    ((0 == strncmp(func, "open_wr", 7)) ||
+		    ((0 == strncmp(func, "access_wr", 7)) ||
+		     (0 == strncmp(func, "open_wr", 7)) ||
 		     (0 == strncmp(func, "creat", 5)) ||
 		     (0 == strncmp(func, "creat64", 7)) ||
 		     (0 == strncmp(func, "mkdir", 5)) ||
@@ -1196,12 +1223,43 @@ static int check_access(sbcontext_t * sbcontext, const char *func, const char *p
 
 			/* No need to check here, as we do it above
 			if (NULL != sbcontext->write_prefixes) { */
+			
+			if (NULL != sbcontext->write_denied_prefixes) {
+				for (i = 0; i < sbcontext->num_write_denied_prefixes; i++) {
+					if (NULL != sbcontext->write_denied_prefixes[i]) {
+						if (0 == strncmp(fpath,
+								 sbcontext->write_denied_prefixes[i],
+								 strlen(sbcontext->write_denied_prefixes[i]))) {
+							/* Special paths in writable context that should
+							 * be denied - not implemented yet */
+							result = 0;
+							break;
+						}
+					}
+				}
+			}
+
+			if ((-1 == result) && (NULL != sbcontext->write_prefixes)) {
+				for (i = 0; i < sbcontext->num_write_prefixes; i++) {
+					if (NULL != sbcontext->write_prefixes[i]) {
+						if (0 == strncmp(fpath,
+								 sbcontext->write_prefixes[i],
+								 strlen(sbcontext->write_prefixes[i]))) {
+							/* Falls in a writable path */
+							result = 1;
+							break;
+						}
+					}
+				}
+			}
+
 			/* XXX: Hack to enable us to remove symlinks pointing
 			 * to protected stuff.  First we make sure that the
 			 * passed path is writable, and if so, check if its a
 			 * symlink, and give access only if the resolved path
 			 * of the symlink's parent also have write access. */
-			if (((0 == strncmp(func, "unlink", 6)) ||
+			if ((-1 == result) &&
+			    ((0 == strncmp(func, "unlink", 6)) ||
 			     (0 == strncmp(func, "lchown", 6)) ||
 			     (0 == strncmp(func, "rename", 6))) &&
 			    ((-1 != lstat(path, &st)) && (S_ISLNK(st.st_mode)))) {
@@ -1255,33 +1313,16 @@ static int check_access(sbcontext_t * sbcontext, const char *func, const char *p
 				}
 			}
 unlink_hack_end:
-			
-			if (NULL != sbcontext->write_denied_prefixes) {
-				for (i = 0; i < sbcontext->num_write_denied_prefixes; i++) {
-					if (NULL != sbcontext->write_denied_prefixes[i]) {
-						if (0 == strncmp(fpath,
-								 sbcontext->write_denied_prefixes[i],
-								 strlen(sbcontext->write_denied_prefixes[i]))) {
-							/* Special paths in writable context that should
-							 * be denied - not implemented yet */
-							result = 0;
-							break;
-						}
-					}
-				}
-			}
 
-			if ((-1 == result) && (NULL != sbcontext->write_prefixes)) {
-				for (i = 0; i < sbcontext->num_write_prefixes; i++) {
-					if (NULL != sbcontext->write_prefixes[i]) {
-						if (0 == strncmp(fpath,
-								 sbcontext->write_prefixes[i],
-								 strlen(sbcontext->write_prefixes[i]))) {
-							/* Falls in a writable path */
-							result = 1;
-							break;
-						}
-					}
+			/* XXX: Hack to allow writing to '/proc/self/fd' (bug #91516)
+			 *      It needs to be here, as for each process '/proc/self'
+			 *      will differ ... */
+			if (-1 == result) {
+				char proc_self_fd[SB_PATH_MAX];
+
+				if (NULL != realpath("/proc/self/fd", proc_self_fd)) {
+					if (0 == strncmp(fpath, proc_self_fd, strlen(proc_self_fd)))
+						result = 1;
 				}
 			}
 
@@ -1371,6 +1412,7 @@ static int check_syscall(sbcontext_t * sbcontext, const char *func, const char *
 					unsetenv("SANDBOX_LOG");
 					fprintf(stderr,	"\e[31;01mSECURITY BREACH\033[0m SANDBOX_LOG %s isn't allowed via SANDBOX_WRITE\n", dpath);
 				} else {
+					check_dlsym(open);
 					log_file = true_open(dpath, O_APPEND | O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 					if (log_file >= 0) {
 						write(log_file, buffer, strlen(buffer));
@@ -1398,9 +1440,9 @@ static int check_syscall(sbcontext_t * sbcontext, const char *func, const char *
 					fprintf(stderr, "\e[31;01mSECURITY BREACH\033[0m  SANDBOX_DEBUG_LOG %s isn't allowed by SANDBOX_WRITE.\n",
 						dpath);
 				} else {
-					debug_log_file = true_open(dpath, O_APPEND | O_WRONLY |
-							           O_CREAT, S_IRUSR | S_IWUSR |
-								   S_IRGRP | S_IROTH);
+					check_dlsym(open);
+					debug_log_file = true_open(dpath, O_APPEND | O_WRONLY | O_CREAT,
+					                           S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 					if (debug_log_file >= 0) {
 						write(debug_log_file, buffer, strlen(buffer));
 						close(debug_log_file);
@@ -1458,10 +1500,15 @@ static int before_syscall(const char *func, const char *file)
 	char *write = getenv("SANDBOX_WRITE");
 	char *predict = getenv("SANDBOX_PREDICT");
 
-	if (!strlen(file)) {
+	if (NULL == file || 0 == strlen(file)) {
 		/* The file/directory does not exist */
+#if 0
 		errno = ENOENT;
 		return 0;
+#else
+		/* XXX: We do not care - let the function handle it */
+		return 1;
+#endif
 	}
 
 	if(sb_init == 0) {
@@ -1555,6 +1602,15 @@ static int before_syscall(const char *func, const char *file)
 	return result;
 }
 
+static int before_syscall_access(const char *func, const char *file, int flags)
+{
+	if (flags & W_OK) {
+		return before_syscall("access_wr", file);
+	} else {
+		return before_syscall("access_rd", file);
+	}
+}
+
 static int before_syscall_open_int(const char *func, const char *file, int flags)
 {
 	if ((flags & O_WRONLY) || (flags & O_RDWR)) {
