@@ -11,7 +11,7 @@
 **    Copyright (C) 2001 Geert Bevin, Uwyn, http://www.uwyn.com
 **    Distributed under the terms of the GNU General Public License, v2 or later 
 **    Author : Geert Bevin <gbevin@uwyn.com>
-**  $Header: /var/cvsroot/gentoo-src/sandbox/sandbox.c,v 1.25 2005/05/13 15:23:28 azarah Exp $
+**  $Header$
 */
 
 /* #define _GNU_SOURCE */
@@ -168,6 +168,7 @@ int sandbox_setup(char *argv[], struct sandbox_info_t *sandbox_info)
 		perror(">>> get portage_tmp_dir");
 		return -1;
 	}
+	setenv(ENV_PORTAGE_TMPDIR, sandbox_info->portage_tmp_dir, 1);
 	
 	if (NULL == realpath(VAR_TMPDIR, sandbox_info->var_tmp_dir)) {
 		perror(">>> get var_tmp_dir");
@@ -179,6 +180,7 @@ int sandbox_setup(char *argv[], struct sandbox_info_t *sandbox_info)
 		return -1;
 	}
 	tmp_dir = sandbox_info->tmp_dir;
+	setenv(ENV_TMPDIR, tmp_dir, 1);
 
 	sandbox_info->home_dir = getenv("HOME");
 	if (!sandbox_info->home_dir) {
@@ -359,8 +361,8 @@ void get_sandbox_write_envvar(char *buf, struct sandbox_info_t *sandbox_info)
 	/* these could go into make.globals later on */
 	snprintf(buf, SB_BUF_LEN,
 		 "%s:%s/.gconfd/lock:%s/.bash_history:%s:%s:%s:%s",
-		 "/dev/zero:/dev/fd/:/dev/null:/dev/pts/:"
-		 "/dev/vc/:/dev/pty:/dev/tty:"
+		 "/dev/zero:/dev/null:/dev/fd:/proc/self/fd:/dev/pts/:"
+		 "/dev/vc/:/dev/pty:/dev/tty:/dev/console:"
 		 "/dev/shm/ngpt:/var/log/scrollkeeper.log:"
 		 "/usr/tmp/conftest:/usr/lib/conftest:"
 		 "/usr/lib32/conftest:/usr/lib64/conftest:"
