@@ -3,7 +3,7 @@
  * Distributed under the terms of the GNU General Public License, v2 or later 
  * Author: Brad House <brad@mainstreetsoftworks.com>
  *
- * $Header: /var/cvsroot/gentoo-src/sandbox/sandbox_futils.c,v 1.11 2005/05/13 15:23:28 azarah Exp $
+ * $Header$
  * 
  */
 
@@ -123,7 +123,8 @@ SB_STATIC int get_tmp_dir(char *tmp_dir)
 	if (NULL == realpath(getenv(ENV_TMPDIR) ? getenv(ENV_TMPDIR)
 					      : TMPDIR,
 				tmp_dir)) {
-		return -1;
+		if (NULL == realpath(TMPDIR, tmp_dir))
+			return -1;
 	}
 
 	return 0;
